import typer
import os
from rin_proj1.get_yaml import get_yaml, return_appmode_dict
from rin_proj1.db_insert import read_csv_and_insert_to_db as csv2db
from rin_proj1.db_insert import export_db_to_csv as db2csv
from rin_proj1.logerror import log_error

app = typer.Typer()


# task 3
@app.command()
def set_config(path: str = "config.yaml") -> None:
    """
    Gets path to config file and runs functions csv-to-db and db-to-csv.
    Use with main set-config or main set-config --path path/to/file.yaml
    """
    config_data = get_yaml(path)
    print(config_data, end="\n\n")

    csv_file: str = config_data.get('csv_file', 'data.csv')
    db_file: str = config_data.get('db_file', 'data.db')
    tb_name: str = config_data.get('table_file', 'data')

    csv2db(csv_file, db_file, tb_name)
    db2csv(db_file, tb_name, "../new_csv.csv")

    print(f"Finished execution and created files in {db_file} and new_csv.csv")


# task 4
@app.command()
@log_error
def set_db(path: str) -> None:
    """get custom db path
    :type path: str
    """
    config_data = get_yaml("config.yaml")
    db_file = path

    csv_file: str = config_data.get('csv_file', "data.csv")
    tb_name: str = config_data.get('table_file', "data")

    csv2db(csv_file, db_file, tb_name)
    db2csv(db_file, tb_name, "../new_csv.csv")

    print("Executed without errors")


# task 5
@app.command()
@log_error
def exec_based_on_env() -> None:
    """get files based on environment and execute functions"""
    app_mode = os.environ.get("APP_MODE", "development")    # get the env, if None then defaults to dev
    config = return_appmode_dict(app_mode)

    csv_file: str = config.get('csv_file', 'data.csv')
    db_file: str = config.get('db_file', 'data.db')
    tb_name: str = config.get('table_file', 'data')

    csv2db(csv_file, db_file, tb_name)
    db2csv(db_file, tb_name, "../new_csv.csv")

    print(f"Created files {config['db_file']} and new_csv.csv")


if __name__ == "__main__":
    app()



from typing import Callable
import logging
from functools import wraps
from os import environ
from rin_proj1.get_yaml import get_yaml

log_map = {
    "critical": logging.CRITICAL,
    "error": logging.ERROR,
    "warning": logging.WARNING,
    "info": logging.INFO,
    "debug": logging.DEBUG,
    "": logging.NOTSET
}


def error_branch(e):
    if isinstance(e, FileNotFoundError):
        if "config.yaml" in e.filename:
            logging.error("Config file can't be found in the current directory. "
                          f"Create a config.yaml file or run command in directory "
                          f"containing config.yaml")
        else:
            logging.error(f"{e.filename} can't be accessed. Check if it exists or if it has read "
                          f"permissions enabled.")
    elif isinstance(e, AttributeError):
        logging.error("config.yaml can't be read. Is the config file empty?")
    elif isinstance(e, KeyError):
        if e.args[0] in ['production', 'development', 'testing']:
            logging.error(f"{key_error.args[0]} doesn't exist. Check config file")
    elif isinstance(e, TypeError):
        logging.error("Check values in csv/db file. TypeError encountered.")
    else:
        logging.error(f"Encountered error. Traceback:\n{e}")


def log_error(func) -> Callable[[...], None]:
    @wraps(func)
    def wrapper(*args, **kwargs) -> None:
        try:
            log_lvl = get_yaml("config.yaml").get("LOG_LEVEL")
        except Exception as e:
            error_branch(e)
            return

        try:
            logging.basicConfig(filename='mpj1_errors.log', format='%(asctime)s - %(message)s',
                                encoding='utf-8', level=log_map[log_lvl],
                                )
            func(*args, **kwargs)
        except Exception as e:
            error_branch(e)
            return
    return wrapper



import yaml


def get_yaml(path: str) -> dict:
    with open(path, 'r') as file:
        config_data: dict = yaml.safe_load(file)
    return config_data


def return_appmode_dict(app_mode: str) -> dict:
    set_mode = get_yaml("config.yaml").get(app_mode, {})
    print(f"MODE: {set_mode['validate']}")
    return set_mode


import csv
import sqlite3


# Read data from CSV and insert into SQLite database
def read_csv_and_insert_to_db(csv_file: str, database_file: str, table_name: str) -> None:
    conn = sqlite3.connect(database_file)
    cursor = conn.cursor()

    with open(csv_file, "r") as f:
        csv_reader = csv.DictReader(f)
        headers = csv_reader.fieldnames

        cursor.execute(f"CREATE TABLE IF NOT EXISTS {table_name} ({', '.join(headers)})")

        for row in csv_reader:
            values = [row[header] for header in headers]
            placeholders = ", ".join(["?"] * len(headers))
            cursor.execute(f"INSERT INTO {table_name} VALUES ({placeholders})", values)

    conn.commit()
    conn.close()


# Export data from SQLite database to CSV
def export_db_to_csv(database_file: str, table_name: str, csv_file: str) -> None:
    conn = sqlite3.connect(database_file)
    cursor = conn.cursor()

    cursor.execute(f"SELECT * FROM {table_name}")
    rows = cursor.fetchall()

    with open(csv_file, "w", newline="") as f:
        csv_writer = csv.writer(f)
        csv_writer.writerow([description[0] for description in cursor.description])
        csv_writer.writerows(rows)

    conn.close()
